#!/usr/bin/ruby

class Proceedings

  def initialize(year, site)
    @year = year
    @site = site

    # site should be 'linuxsymposium' or 'gccsummit'
    @fullsite = 'www.' + site + '.org'
    if site =~ /gcc/i
      @tla = 'GCC'
      @fullname = 'GCC Summit'
    else
      @tla = 'OLS'
      @fullname = 'Linux Symposium'
    end
    @papers  = Array.new
    @authors = Array.new
    @titles  = Array.new
    @ckeys   = Array.new
    @bios    = Array.new
    @abstract_url = 'http://' + @fullsite + '/' + @year.to_s + '/view_abstract.php?content_key='
  end

  def tla
    @tla
  end

  def authors
    @authors
  end
  def abstract_url
    @abstract_url
  end
  def papers
    @papers
  end
  def titles
    @titles
  end
  def ckeys
    @ckeys
  end
  def bios
    @bios
  end
  def author(id)
    @authors[id.to_i]
  end
  def paper(id)
    @papers[id.to_i]
  end
  def title(id)
    @titles[id.to_i]
  end
  def ckey(id)
    @ckeys[id.to_i]
  end
  def bio(id)
    @bios[id.to_i]
  end
  def year
    @year
  end
  def site
    @site
  end

  def find_author(au)
    zau = au.to_s
    @papers.find_all {|p| p.author.to_s == zau }
  end
  def find_paper(id)
    zid = id.to_i
    @papers.find_all {|p| p.id.to_i == zid }
  end
  def find_title(tit)
    ztit = tit.to_s
    @papers.find_all {|p| p.title.to_s == ztit }
  end
  def find_dirname(d)
    zd = d.to_s
    @papers.find_all {|p| p.dir.to_s == zd }
  end

  def delete_paper(id)
    p = self.find_paper(id)
    ret = nil
    if p
      ret = @papers.delete(p)
    end
    ret
  end

  def load(fname)
    @stuff = `egrep '^[_A-Z]+=' #{fname}`
    title_re = Regexp.new('^TITLE="([^"]+)"')
    bio_re = Regexp.new('^BIO_ID="([^"]+)"')
    ckey_re = Regexp.new('^CONTENT_KEY="([^"]+)"')
    author_re = Regexp.new('^AUTHOR="([^"]+)"')
    recd_re = Regexp.new('^RECEIVED=(\d)')
    id_re = Regexp.new('^PAPER_ID=(\d+)')
    authdir_re = Regexp.new('^AUTHDIR=(.*)')
    @titles = @stuff.grep(title_re) {|s| s[title_re,1]}
    @bios = @stuff.grep(bio_re) {|s| s[bio_re,1]}
    @ckeys = @stuff.grep(ckey_re) {|s| s[ckey_re,1]}
    @authors = @stuff.grep(author_re) {|s| s[author_re,1]}
    @recds = @stuff.grep(recd_re) {|s| s[recd_re,1]}
    @ids = @stuff.grep(id_re) {|s| s[id_re,1]}
    @dirnames = @stuff.grep(authdir_re) {|s| s[authdir_re,1]}
    # STDERR.puts "got #{@titles.size} titles"
    if @titles.size == @bios.size and
        @authors.size == @recds.size and
        @ids.size == @dirnames.size and
        @titles.size == @authors.size and
        @titles.size == @ids.size
      i = 0
      @titles.each do |t|
        r = false
        r = true if (@recds[i].to_i != 0)
        @papers << Paper.new(@authors[i], t, @bios[i], @ckeys[i], @ids[i], r, @dirnames[i])
        # STDERR.puts "added paper #{i} by author #{@authors[i]}"
        i += 1
      end
    else
      STDERR.puts "failed size check on load of #{fname}"
    end
  end
  def save(fname, mod="w+", bcmd=nil)
    File.open(fname, mod) {|f|
      i = 0
      @papers.each do |p|
        f.print('PAPER_ID=' + p.id.to_s + "\n")
        f.print('TITLE="' + p.title + '"' + "\n")
        f.print('BIO_ID="' + p.bio + '"' + "\n")
        f.print('CONTENT_KEY="' + p.ckey + '"' + "\n")
        f.print('AUTHOR="' + p.author + '"' + "\n")
        f.print('AUTHDIR=' + p.dir + "\n")
        f.print('RECEIVED=' + (p.recd? ? "1" : "0") + "\n\n")
        if bcmd
          f.print(bcmd + "\n\n")
        end
      end
    }
  end

  def add_paper_id(id)
    p = nil
    if (! id.between?(0,(@authors.size - 1))) and (nil == @authors[id])
      p = Papers.new(@authors[id], @titles[id], @bios[id], @ckeys[id], id)
      @papers << p if (p)
    end
    p
  end

  def fetch
    url = 'http://www.' + @site + '.org/' + @year.to_s + '/speakers.php?type=TALK'
    @scrape = `wget -qO- '#{url}'`
    # STDERR.puts @scrape
    # STDERR.puts "got #{@scrape.size} objects out of webscrape of '#{url}'"
    make_titlesAuthorsKeysBios
    if @titles.size == @bios.size and
        @authors.size == @ckeys.size
        @titles.size == @authors.size
      i = 0
      @titles.each do |t|
        @papers << Paper.new(@authors[i], t, @bios[i], @ckeys[i], i+1, false)
        i += 1
      end
    else
      STDERR.puts "failed size check on fetch, insert debugging here"
    end
    fix_dirnames
  end

  def fix_dirnames
    @papers.each do |p|
      slush = find_dirname(p.dir)
      if slush.size > 1
        i = 1
        slush.each do |s|
          dn = s.dir + i.to_s
          s.set_dirname(dn)
          i += 1
        end
      end
    end
  end

  def make_titlesAuthorsKeysBios
    author_re = Regexp.new('^\s+<td><a[^>]+>([^<]+)</a></td>')
    bio_re = Regexp.new('=(\d+)')
    key_re = Regexp.new('content_key=(\d+).>')
    title_re = Regexp.new('<a href=[^>]+>(.*)</a>.*')
    flag = false
    @scrape.each do |l|
      l.chomp!
      if flag
        @titles << (l[title_re,1] or l).strip.gsub(/\s\s+/, " ").gsub(/&quot;/,"'").gsub(/ - /,'---')
        @ckeys  << (l[key_re,1] or "0").strip
        flag = false
      else
        flag = true if l.match(/colspan=.2/)
      end
    end
    @scrape.grep(/view_bio/).each do |b|
      @authors << b[author_re,1].strip.gsub(/\s\s+/, " ")
      @bios << b[bio_re,1].strip
    end
    # STDERR.puts "titles: #{@titles.size}"
  end


end

# This could really just be a hash for now, but perhaps there
# are classier things that can be done with a Paper...
class Paper
  def initialize(author, title, bio, ckey, id, recd=false, dirname=nil)
    @author = sanitize(author.strip)
    @title = sanitize(title.strip)
    @bio = bio
    @ckey = ckey
    @id = id
    if dirname
      @dirname = dirname
    else
      # Consider input such as: "Aneesh Kumar K.V"
      @dirname = make_dirname(author.gsub(/\s\S\.\S\s*/, " ").split(/\s+/)[-1])
    end
    @recd = recd
  end

  def sanitize(s)
    # turn crap like "don&#039;t" into "don't"
    ret = s
    magic = Regexp.new('(&#(\d+);)')
    bad = s[magic,1]
    if bad
      n = s[magic,2]
      if n
        ret = s.gsub(magic, n.to_i.chr)
      end
    end
    ret
  end

  def convChar(c)
    case
    when c.between?(65,90) : return c.chr.downcase
    when c.between?(97,122) : return c.chr
    when c.between?(192, 197) : return "a"
    when c.between?(224, 229) : return "a"
    when c.between?(200, 203) : return "e"
    when c.between?(232, 235) : return "e"
    when c.between?(204, 207) : return "i"
    when c.between?(236, 239) : return "i"
    when (c.between?(210, 214) or c == 216 or c.between?(242,248)) : return "o"
    when (c.between?(217,220) or c.between?(249,252)) : return "u"
    when [221,253,255].member?(c) : return "y"
    when [199,231].member?(c) : return "c"
    when [215,247].member?(c) : return "x"
    when [222,254].member?(c) : return "p"
    when [208,240].member?(c) : return "d"
    when [209,241].member?(c) : return "n"
    when [198,230].member?(c) : return "ae"
    when c == 223 : return "ss"
    end
    return ""
  end

  def make_dirname(arg)
    # Make a directory name that's friendly to LANG=C:
    # all lowercase, no spaces/punct/unprintables, no chars > 0x7F.
    out = ''
    arg.each_byte {|b| out += convChar(b) }
    return out
  end

  def set_dirname(arg)
    @dirname = arg
  end

  def author
    @author
  end
  def dir
    @dirname
  end
  def title
    @title
  end
  def ckey
    @ckey
  end
  def bio
    @bio
  end
  def id
    @id
  end
  def recd?
    @recd
  end
  def recd(val)
    @recd = val
  end

  alias received? recd?
  alias received recd
  alias dirname dir

end
